@@ -86,14 +86,14 @@ struct _XimCacheStruct {
     /* char encoding[1] */
 };
 
-Private struct  _XimCacheStruct* _XimCache_mmap = NULL;
-Private DefTreeBase _XimCachedDefaultTreeBase;
-Private int     _XimCachedDefaultTreeRefcount = 0;
+static struct  _XimCacheStruct* _XimCache_mmap = NULL;
+static DefTreeBase _XimCachedDefaultTreeBase;
+static int     _XimCachedDefaultTreeRefcount = 0;
 
 #endif
 
 
-Public Bool
+Bool
 _XimCheckIfLocalProcessing(Xim im)
 {
     FILE        *fp;
@@ -117,7 +117,7 @@ _XimCheckIfLocalProcessing(Xim im)
     return(False);
 }
 
-Private void
+static void
 XimFreeDefaultTree(
     DefTreeBase *b)
 {
@@ -144,7 +144,7 @@ XimFreeDefaultTree(
     b->utf8used = b->utf8size = 0;
 }
 
-Public void
+void
 _XimLocalIMFree(
     Xim		im)
 {
@@ -218,7 +218,7 @@ _XimLocalIMFree(
     return;
 }
 
-Private Status
+static Status
 _XimLocalCloseIM(
     XIM		xim)
 {
@@ -231,7 +231,7 @@ _XimLocalCloseIM(
     while (ic) {
 	(*ic->methods->destroy) (ic);
 	next = ic->core.next;
-	Xfree ((char *) ic);
+	Xfree (ic);
 	ic = next;
     }
     _XimLocalIMFree(im);
@@ -239,7 +239,7 @@ _XimLocalCloseIM(
     return(True);
 }
 
-Public char *
+char *
 _XimLocalGetIMValues(
     XIM			 xim,
     XIMArg		*values)
@@ -252,7 +252,7 @@ _XimLocalGetIMValues(
 			im->core.im_resources, im->core.im_num_resources));
 }
 
-Public char *
+char *
 _XimLocalSetIMValues(
     XIM			 xim,
     XIMArg		*values)
@@ -271,7 +271,7 @@ _XimLocalSetIMValues(
 
 #ifdef COMPOSECACHE
 
-Private Bool
+static Bool
 _XimReadCachedDefaultTree(
     int          fd_cache,
     const char  *name,
@@ -322,7 +322,7 @@ _XimReadCachedDefaultTree(
     return True;
 }
 
-Private unsigned int strToHash (
+static unsigned int strToHash (
     const char *name)
 {
     unsigned int hash = 0;
@@ -334,7 +334,7 @@ Private unsigned int strToHash (
 
 /* Returns read-only fd of cache file, -1 if none.
  * Sets *res to cache filename if safe. Sets *size to file size of cache. */
-Private int _XimCachedFileName (
+static int _XimCachedFileName (
     const char *dir, const char *name,
     const char *intname, const char *encoding,
     uid_t uid, int isglobal, char **res, off_t *size)
@@ -419,7 +419,7 @@ Private int _XimCachedFileName (
 }
 
 
-Private Bool _XimLoadCache (
+static Bool _XimLoadCache (
     int         fd,
     const char *name,
     const char *encoding,
@@ -439,7 +439,7 @@ Private Bool _XimLoadCache (
 }
 
 
-Private void
+static void
 _XimWriteCachedDefaultTree(
     const char *name,
     const char *encoding,
@@ -489,10 +489,13 @@ _XimWriteCachedDefaultTree(
 
     /* This STILL might be racy on NFS */
     if ( (fd = _XOpenFileMode (cachename, O_WRONLY | O_CREAT | O_EXCL,
-			       0600)) < 0)
+			       0600)) < 0) {
+       Xfree(m);
        return;
+    }
     if (! (fp = fdopen (fd, "wb")) ) {
        close (fd);
+       Xfree(m);
        return;
     }
     fwrite (m, msize, 1, fp);
@@ -511,7 +514,7 @@ _XimWriteCachedDefaultTree(
 #endif
 
 
-Private void
+static void
 _XimCreateDefaultTree(
     Xim		im)
 {
@@ -641,7 +644,7 @@ _XimCreateDefaultTree(
 	Xfree (cachename);
 }
 
-Private XIMMethodsRec      Xim_im_local_methods = {
+static XIMMethodsRec      Xim_im_local_methods = {
     _XimLocalCloseIM,           /* close */
     _XimLocalSetIMValues,       /* set_values */
     _XimLocalGetIMValues,       /* get_values */
@@ -651,7 +654,7 @@ Private XIMMethodsRec      Xim_im_local_methods = {
     _XimLcctstoutf8		/* ctstoutf8 */
 };
 
-Public Bool
+Bool
 _XimLocalOpenIM(
     Xim			 im)
 {
