@@ -152,7 +152,7 @@ Bool _XPollfdCacheInit(
 #ifdef USE_POLL
     struct pollfd *pfp;
 
-    pfp = (struct pollfd *)Xmalloc(POLLFD_CACHE_SIZE * sizeof(struct pollfd));
+    pfp = Xmalloc(POLLFD_CACHE_SIZE * sizeof(struct pollfd));
     if (!pfp)
 	return False;
     pfp[0].fd = dpy->fd;
@@ -239,7 +239,7 @@ void _XSeqSyncFunction(
 static int
 _XPrivSyncFunction (Display *dpy)
 {
-#if XTHREADS
+#ifdef XTHREADS
     assert(!dpy->lock_fns);
 #endif
     assert(dpy->synchandler == _XPrivSyncFunction);
@@ -293,124 +293,6 @@ void _XRead32(
 }
 #endif /* LONG64 */
 
-#ifdef WORD64
-
-/*
- * XXX This is a *really* stupid way of doing this....
- * PACKBUFFERSIZE must be a multiple of 4.
- */
-
-#define PACKBUFFERSIZE 4096
-
-
-/*
- * _XRead32 - Read bytes from the socket unpacking each 32 bits
- *            into a long (64 bits on a CRAY computer).
- *
- */
-static void _doXRead32(
-        register Display *dpy,
-        register long *data
-        register long size,
-	register char *packbuffer)
-{
- long *lpack,*lp;
- long mask32 = 0x00000000ffffffff;
- long maskw, nwords, i, bits;
-
-        _XReadPad (dpy, packbuffer, size);
-
-        lp = data;
-        lpack = (long *) packbuffer;
-        nwords = size >> 2;
-        bits = 32;
-
-        for(i=0;i<nwords;i++){
-            maskw = mask32 << bits;
-           *lp++ = ( *lpack & maskw ) >> bits;
-            bits = bits ^32;
-            if(bits){
-               lpack++;
-            }
-        }
-}
-
-void _XRead32(
-    Display *dpy,
-    long *data,
-    long len)
-{
-    char packbuffer[PACKBUFFERSIZE];
-    unsigned nunits = PACKBUFFERSIZE >> 2;
-
-    for (; len > PACKBUFFERSIZE; len -= PACKBUFFERSIZE, data += nunits) {
-	_doXRead32 (dpy, data, PACKBUFFERSIZE, packbuffer);
-    }
-    if (len) _doXRead32 (dpy, data, len, packbuffer);
-}
-
-
-
-/*
- * _XRead16 - Read bytes from the socket unpacking each 16 bits
- *            into a long (64 bits on a CRAY computer).
- *
- */
-static _doXRead16(
-        register Display *dpy,
-        register short *data,
-        register long size,
-	char *packbuffer)
-{
-	long *lpack,*lp;
-	long mask16 = 0x000000000000ffff;
-	long maskw, nwords, i, bits;
-
-        (void) _XRead(dpy,packbuffer,size);	/* don't do a padded read... */
-
-        lp = (long *) data;
-        lpack = (long *) packbuffer;
-        nwords = size >> 1;  /* number of 16 bit words to be unpacked */
-        bits = 48;
-        for(i=0;i<nwords;i++){
-            maskw = mask16 << bits;
-           *lp++ = ( *lpack & maskw ) >> bits;
-            bits -= 16;
-            if(bits < 0){
-               lpack++;
-               bits = 48;
-            }
-        }
-}
-
-void _XRead16(
-    Display *dpy,
-    short *data,
-    long len)
-{
-    char packbuffer[PACKBUFFERSIZE];
-    unsigned nunits = PACKBUFFERSIZE >> 1;
-
-    for (; len > PACKBUFFERSIZE; len -= PACKBUFFERSIZE, data += nunits) {
-	_doXRead16 (dpy, data, PACKBUFFERSIZE, packbuffer);
-    }
-    if (len) _doXRead16 (dpy, data, len, packbuffer);
-}
-
-void _XRead16Pad(
-    Display *dpy,
-    short *data,
-    long size)
-{
-    int slop = (size & 3);
-    short slopbuf[3];
-
-    _XRead16 (dpy, data, size);
-    if (slop > 0) {
-	_XRead16 (dpy, slopbuf, 4 - slop);
-    }
-}
-#endif /* WORD64 */
 
 /*
  * The hard part about this is that we only get 16 bits from a reply.
@@ -492,10 +374,10 @@ _XRegisterInternalConnection(
     struct _XConnWatchInfo *watchers;
     XPointer *wd;
 
-    new_conni = (struct _XConnectionInfo*)Xmalloc(sizeof(struct _XConnectionInfo));
+    new_conni = Xmalloc(sizeof(struct _XConnectionInfo));
     if (!new_conni)
 	return 0;
-    new_conni->watch_data = (XPointer *)Xmalloc(dpy->watcher_count * sizeof(XPointer));
+    new_conni->watch_data = Xmalloc(dpy->watcher_count * sizeof(XPointer));
     if (!new_conni->watch_data) {
 	Xfree(new_conni);
 	return 0;
@@ -582,7 +464,7 @@ XInternalConnectionNumbers(
     count = 0;
     for (info_list=dpy->im_fd_info; info_list; info_list=info_list->next)
 	count++;
-    fd_list = (int*) Xmalloc (count * sizeof(int));
+    fd_list = Xmalloc (count * sizeof(int));
     if (!fd_list) {
 	UnlockDisplay(dpy);
 	return 0;
@@ -655,9 +537,8 @@ XAddConnectionWatch(
 
     /* allocate new watch data */
     for (info_list=dpy->im_fd_info; info_list; info_list=info_list->next) {
-	wd_array = (XPointer *)Xrealloc((char *)info_list->watch_data,
-					(dpy->watcher_count + 1) *
-					sizeof(XPointer));
+	wd_array = Xrealloc(info_list->watch_data,
+			    (dpy->watcher_count + 1) * sizeof(XPointer));
 	if (!wd_array) {
 	    UnlockDisplay(dpy);
 	    return 0;
@@ -666,7 +547,7 @@ XAddConnectionWatch(
 	wd_array[dpy->watcher_count] = NULL;	/* for cleanliness */
     }
 
-    new_watcher = (struct _XConnWatchInfo*)Xmalloc(sizeof(struct _XConnWatchInfo));
+    new_watcher = Xmalloc(sizeof(struct _XConnWatchInfo));
     if (!new_watcher) {
 	UnlockDisplay(dpy);
 	return 0;
@@ -874,8 +755,7 @@ void _XEnq(
 		/* If dpy->qfree is non-NULL do this, else malloc a new one. */
 		dpy->qfree = qelt->next;
 	}
-	else if ((qelt =
-	    (_XQEvent *) Xmalloc((unsigned)sizeof(_XQEvent))) == NULL) {
+	else if ((qelt = Xmalloc(sizeof(_XQEvent))) == NULL) {
 		/* Malloc call failed! */
 		ESET(ENOMEM);
 		_XIOError(dpy);
@@ -886,7 +766,7 @@ void _XEnq(
 	extension = ((xGenericEvent*)event)->extension;
 
 	qelt->event.type = type;
-	/* If an extension has registerd a generic_event_vec handler, then
+	/* If an extension has registered a generic_event_vec handler, then
 	 * it can handle event cookies. Otherwise, proceed with the normal
 	 * event handlers.
 	 *
@@ -1432,7 +1312,7 @@ static int _XPrintDefaultError(
 	mesg, BUFSIZ);
     (void) fprintf(fp, mesg, event->request_code);
     if (event->request_code < 128) {
-	sprintf(number, "%d", event->request_code);
+	snprintf(number, sizeof(number), "%d", event->request_code);
 	XGetErrorDatabaseText(dpy, "XRequest", number, "", buffer, BUFSIZ);
     } else {
 	for (ext = dpy->ext_procs;
@@ -1452,7 +1332,7 @@ static int _XPrintDefaultError(
 	fputs("  ", fp);
 	(void) fprintf(fp, mesg, event->minor_code);
 	if (ext) {
-	    sprintf(mesg, "%s.%d", ext->name, event->minor_code);
+	    snprintf(mesg, sizeof(mesg), "%s.%d", ext->name, event->minor_code);
 	    XGetErrorDatabaseText(dpy, "XRequest", mesg, "", buffer, BUFSIZ);
 	    (void) fprintf(fp, " (%s)", buffer);
 	}
@@ -1475,8 +1355,8 @@ static int _XPrintDefaultError(
 		bext = ext;
 	}
 	if (bext)
-	    sprintf(buffer, "%s.%d", bext->name,
-		    event->error_code - bext->codes.first_error);
+	    snprintf(buffer, sizeof(buffer), "%s.%d", bext->name,
+                     event->error_code - bext->codes.first_error);
 	else
 	    strcpy(buffer, "Value");
 	XGetErrorDatabaseText(dpy, mtype, buffer, "", mesg, BUFSIZ);
@@ -1636,7 +1516,7 @@ char *_XAllocScratch(
 {
 	if (nbytes > dpy->scratch_length) {
 	    if (dpy->scratch_buffer) Xfree (dpy->scratch_buffer);
-	    if ((dpy->scratch_buffer = Xmalloc((unsigned) nbytes)))
+	    if ((dpy->scratch_buffer = Xmalloc(nbytes)))
 		dpy->scratch_length = nbytes;
 	    else dpy->scratch_length = 0;
 	}
@@ -1722,7 +1602,7 @@ void _Xbcopy(b1, b2, length)
 #ifdef DataRoutineIsProcedure
 void Data(
 	Display *dpy,
-	char *data,
+	_Xconst char *data,
 	long len)
 {
 	if (dpy->bufptr + (len) <= dpy->bufmax) {
@@ -1739,7 +1619,7 @@ void Data(
 int
 _XData32(
     Display *dpy,
-    register long *data,
+    register _Xconst long *data,
     unsigned len)
 {
     register int *buf;
@@ -1764,122 +1644,6 @@ _XData32(
 }
 #endif /* LONG64 */
 
-#ifdef WORD64
-
-/*
- * XXX This is a *really* stupid way of doing this.  It should just use
- * dpy->bufptr directly, taking into account where in the word it is.
- */
-
-/*
- * Data16 - Place 16 bit data in the buffer.
- *
- * "dpy" is a pointer to a Display.
- * "data" is a pointer to the data.
- * "len" is the length in bytes of the data.
- */
-
-static doData16(
-    register Display *dpy,
-    short *data,
-    unsigned len,
-    char *packbuffer)
-{
-    long *lp,*lpack;
-    long i, nwords,bits;
-    long mask16 = 0x000000000000ffff;
-
-        lp = (long *)data;
-        lpack = (long *)packbuffer;
-
-/*  nwords is the number of 16 bit values to be packed,
- *  the low order 16 bits of each word will be packed
- *  into 64 bit words
- */
-        nwords = len >> 1;
-        bits = 48;
-
-        for(i=0;i<nwords;i++){
-	   if (bits == 48) *lpack = 0;
-           *lpack ^= (*lp & mask16) << bits;
-           bits -= 16 ;
-           lp++;
-           if(bits < 0){
-               lpack++;
-               bits = 48;
-           }
-        }
-        Data(dpy, packbuffer, len);
-}
-
-_XData16 (
-    Display *dpy,
-    short *data,
-    unsigned len)
-{
-    char packbuffer[PACKBUFFERSIZE];
-    unsigned nunits = PACKBUFFERSIZE >> 1;
-
-    for (; len > PACKBUFFERSIZE; len -= PACKBUFFERSIZE, data += nunits) {
-	doData16 (dpy, data, PACKBUFFERSIZE, packbuffer);
-    }
-    if (len) doData16 (dpy, data, len, packbuffer);
-}
-
-/*
- * Data32 - Place 32 bit data in the buffer.
- *
- * "dpy" is a pointer to a Display.
- * "data" is a pointer to the data.
- * "len" is the length in bytes of the data.
- */
-
-static doData32(
-    register Display *dpy
-    long *data,
-    unsigned len,
-    char *packbuffer)
-{
-    long *lp,*lpack;
-    long i,bits,nwords;
-    long mask32 = 0x00000000ffffffff;
-
-        lpack = (long *) packbuffer;
-        lp = data;
-
-/*  nwords is the number of 32 bit values to be packed
- *  the low order 32 bits of each word will be packed
- *  into 64 bit words
- */
-        nwords = len >> 2;
-        bits = 32;
-
-        for(i=0;i<nwords;i++){
-	   if (bits == 32) *lpack = 0;
-           *lpack ^= (*lp & mask32) << bits;
-           bits = bits ^32;
-           lp++;
-           if(bits)
-              lpack++;
-        }
-        Data(dpy, packbuffer, len);
-}
-
-void _XData32(
-    Display *dpy,
-    long *data,
-    unsigned len)
-{
-    char packbuffer[PACKBUFFERSIZE];
-    unsigned nunits = PACKBUFFERSIZE >> 2;
-
-    for (; len > PACKBUFFERSIZE; len -= PACKBUFFERSIZE, data += nunits) {
-	doData32 (dpy, data, PACKBUFFERSIZE, packbuffer);
-    }
-    if (len) doData32 (dpy, data, len, packbuffer);
-}
-
-#endif /* WORD64 */
 
 
 /* Make sure this produces the same string as DefineLocal/DefineSelf in xdm.
@@ -1956,6 +1720,43 @@ Screen *_XScreenOfWindow(Display *dpy, Window w)
 }
 
 
+/*
+ * WARNING: This implementation's pre-conditions and post-conditions
+ * must remain compatible with the old macro-based implementations of
+ * GetReq, GetReqExtra, GetResReq, and GetEmptyReq. The portions of the
+ * Display structure affected by those macros are part of libX11's
+ * ABI.
+ */
+void *_XGetRequest(Display *dpy, CARD8 type, size_t len)
+{
+    xReq *req;
+
+    if (dpy->bufptr + len > dpy->bufmax)
+	_XFlush(dpy);
+    /* Request still too large, so do not allow it to overflow. */
+    if (dpy->bufptr + len > dpy->bufmax) {
+	fprintf(stderr,
+		"Xlib: request %d length %zd would exceed buffer size.\n",
+		type, len);
+	/* Changes failure condition from overflow to NULL dereference. */
+	return NULL;
+    }
+
+    if (len % 4)
+	fprintf(stderr,
+		"Xlib: request %d length %zd not a multiple of 4.\n",
+		type, len);
+
+    dpy->last_req = dpy->bufptr;
+
+    req = (xReq*)dpy->bufptr;
+    req->reqType = type;
+    req->length = len / 4;
+    dpy->bufptr += len;
+    dpy->request++;
+    return req;
+}
+
 #if defined(WIN32)
 
 /*
